<?php

/**
 * Copyright (c) 2016, BitKassa B.V.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * @category    BitKassa
 * @package     BitKassa_Bitcoin
 * @author      BitKassa (info@bitkassa.nl) / Mollie B.V. (info@mollie.nl)
 * @version     v1.0 (based on Mollie_Ideal v4.7)
 * @copyright   Copyright (c) 2012 Mollie B.V. (http://www.mollie.nl)
 * @license     http://www.opensource.org/licenses/bsd-license.php  Berkeley Software Distribution License (BSD-License 2)
 *
 **/

require_once('bitcoin.class.php');

class ControllerPaymentBitKassaBitcoin extends Controller {
	/**
	 * @param $bitkassa_bitcoin_merchantkey
	 * @codeCoverageIgnore
	 * @return Bitcoin_Payment
	 */
	protected function getBitcoinPaymentObject($bitkassa_bitcoin_merchantkey) {
		return new Bitcoin_Payment($bitkassa_bitcoin_merchantkey);
	}

	/**
	 * This gets called by OpenCart at the checkout page and generates the paymentmethod
	 */
	public function index() {

		// Haal bitcoinprijs op
		$this->data['btcamount'] = number_format($this->getOrderBitcoinPrice(),8,$this->language->get('decimal_point'),$this->language->get('thousand_point'));

		// Create Bitcoin object
		$bitcoin = $this->getBitcoinPaymentObject($this->config->get('bitkassa_bitcoin_merchantkey'));

		// Set template data
		$this->data['button_confirm'] = $this->language->get('button_confirm');
		$this->data['action']         = $this->url->link('payment/bitkassa_bitcoin/payment', '', 'SSL');

		// Check if view is at default template else use modified template path
		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/bitkassa_bitcoin_start.tpl')) {
			$this->template = $this->config->get('config_template') . '/template/payment/bitkassa_bitcoin_start.tpl';
		} else {
			$this->template = 'default/template/payment/bitkassa_bitcoin_start.tpl';
		}

		// Render HTML output
		$this->render();
	}

	/**
	 * The payment action creates the Bitcoin payment and redirects the customer to BitKassa
	 */
	public function payment() {

		if ($this->request->server['REQUEST_METHOD'] == 'POST') {
			// Load essentials
			$this->load->model('checkout/order');
			$this->load->model('payment/bitkassa_bitcoin');
			$this->load->language('payment/bitkassa_bitcoin');

			// Create bitcoin object
			$bitcoin = $this->getBitcoinPaymentObject($this->config->get('bitkassa_bitcoin_merchantkey'));

			if (isset($this->request->post['transaction_id'])) {
				// Load failed order and payment
				$payment = $this->model_payment_bitkassa_bitcoin->getPaymentById($this->request->post['transaction_id']);
				$order   = $this->model_payment_bitkassa_bitcoin->getOrderById($payment['order_id']);
			} else {
				// Load last order from session
				$order = $this->model_payment_bitkassa_bitcoin->getOrderById($this->session->data['order_id']);
			}

			// Assign required vars for createPayment
			$amount      = intval(round($order['total'] * 100));
			$description = str_replace('%', $order['order_id'], html_entity_decode($this->config->get('bitkassa_bitcoin_description'), ENT_QUOTES, 'UTF-8'));
			$return_url  = $this->url->link('payment/bitkassa_bitcoin/status', '', 'SSL');
			$report_url  = $this->url->link('payment/bitkassa_bitcoin/report', '', 'SSL');

			try {
				// Create the payment, if succeeded confirm the order and redirect the customer to BitKassa
				if ($bitcoin->createPayment($amount, $description, $return_url, $report_url)) {
					if (isset($this->request->post['transaction_id'])) {
						$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_pending_status_id'), $this->language->get('text_redirected'), FALSE);
					} else {
						$this->model_checkout_order->confirm($order['order_id'], $this->config->get('bitkassa_bitcoin_pending_status_id'), $this->language->get('text_redirected'), FALSE);
					}

					$this->model_payment_bitkassa_bitcoin->setPayment($order['order_id'], $bitcoin->getTransactionId());
					$this->redirect($bitcoin->getBitKassaURL());
				} else {
					throw new Exception($bitcoin->getErrorMessage());
				}
			} catch (Exception $e) {
				echo("Kon geen betaling aanmaken, neem contact op met de beheerder.<br /><br/>
					Foutmelding voor de beheerder: " . $e->getMessage()
				);

				global $log;

				if ($this->config->get('config_error_log')) {
					$log->write('PHP ' . $e->getCode() . ':  ' . $e->getMessage() . ' in ' . __FILE__ . ' on line ' . __LINE__);
				}
			}
		}
	}

	/**
	 * This action is getting called by BitKassa to report the payment status
	 */
	public function report () {
		if (!empty($this->request->get['transaction_id'])) {
			// Create bitcoin object
			$bitcoin = $this->getBitcoinPaymentObject($this->config->get('bitkassa_bitcoin_merchantkey'));

			// Get transaction_id from URL
			$transaction_id = $this->request->get['transaction_id'];

			// Check payment
			$bitcoin->checkPayment($transaction_id);

			// Load essentials
			$this->load->model('checkout/order');
			$this->load->model('payment/bitkassa_bitcoin');
			$this->load->language('payment/bitkassa_bitcoin');

			//Get order_id of this transaction from db
			$payment  = $this->model_payment_bitkassa_bitcoin->getPaymentById($transaction_id);
			$order    = $this->model_payment_bitkassa_bitcoin->getOrderById($payment['order_id']);
			$consumer = null;

			if (!empty($order)) {
				// Only update if the transaction is in 'pending' or 'paid' status
				if ( $order['order_status_id'] == $this->config->get('bitkassa_bitcoin_pending_status_id') && $bitcoin->getStatus() != ModelPaymentBitKassaBitcoin::STATUS_PENDING ||
					 $order['order_status_id'] == $this->config->get('bitkassa_bitcoin_paid_status_id') && $bitcoin->getStatus() != ModelPaymentBitKassaBitcoin::STATUS_PAID ) {
					$amount = intval(round($order['total'] * 100));
					$correct_amount = ($bitcoin->getAmount() >= $order['total']-1);

					switch ($bitcoin->getStatus()) {
						case ModelPaymentBitKassaBitcoin::STATUS_CONFIRMED:
							$consumer = $bitcoin->getConsumerInfo();
							if ($correct_amount)
								$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_confirmed_status_id'), $this->language->get('response_confirmed'), TRUE); // Confirmed
							else
								$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_failed_status_id'), $this->language->get('response_fraud'), FALSE); // Fraude
							break;
						case ModelPaymentBitKassaBitcoin::STATUS_PAID:
							$consumer = $bitcoin->getConsumerInfo();
							$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_paid_status_id'), $this->language->get('response_paid'), TRUE); // Processing
							break;
						case ModelPaymentBitKassaBitcoin::STATUS_CANCELLED:
							$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_cancelled_status_id'), $this->language->get('response_cancelled'), FALSE); // Cancelled
							break;
						case ModelPaymentBitKassaBitcoin::STATUS_FAILURE:
							$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_failed_status_id'), $this->language->get('response_failed'), TRUE); // Fail
							break;
						case ModelPaymentBitKassaBitcoin::STATUS_EXPIRED:
							$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_expired_status_id'), $this->language->get('response_expired'), FALSE); // Expired
							break;
						default:
							$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_failed_status_id'), $this->language->get('response_unknown'), FALSE); // Fail
							break;
					}

					// update bitkassa status
					$this->model_payment_bitkassa_bitcoin->updatePayment($payment['transaction_id'], $bitcoin->getStatus(), $consumer);
				} else {
					// same status, do nothing?
					//$this->model_checkout_order->update($order['order_id'], $this->config->get('bitkassa_bitcoin_failed_status_id'), $this->language->get('response_failed'), TRUE); // Fail
				}
			}
		}
	}

	/**
	 * Customer returning from BitKassa with a transaction_id
	 * Depending on what the state of the payment is they get redirected to the corresponding page
	 */
	public function status () {
		if (!empty($this->request->get['transaction_id'])) {

			// transaction id
			$transaction_id = $this->request->get['transaction_id'];

			// Create bitcoin object
			$bitcoin = $this->getBitcoinPaymentObject($this->config->get('bitkassa_bitcoin_merchantkey'));

			// Load essential settings
			$this->load->model('payment/bitkassa_bitcoin');
			$this->load->language('payment/bitkassa_bitcoin');

			$payment = $this->model_payment_bitkassa_bitcoin->getPaymentById($transaction_id);

			/*
			 * Now that the customer has returned to our web site, check if we already know if the payment has
			 * succeeded. It the payment is all good, we need to clear the cart.
			 */
			if (isset($payment['status']) &&
				($payment['status'] == ModelPaymentBitKassaBitcoin::STATUS_PAID || $payment['status'] == ModelPaymentBitKassaBitcoin::STATUS_CONFIRMED)) {
				/** @var $this->cart Cart */
				$this->cart->clear();
			}

			$order = $this->model_payment_bitkassa_bitcoin->getOrderById($payment['order_id']);

			// Set template data
			$this->document->setTitle($this->language->get('bitcoin_title'));
			$this->data['payment'] = $payment;
			$this->data['order']   = $order;
			$this->data['message'] = $this->language;
			$this->data['action']  = $this->url->link('payment/bitkassa_bitcoin/payment', '', 'SSL');
			// Haal bitcoinprijs op
			$this->data['btcamount'] = number_format($this->getOrderBitcoinPrice(),8,$this->language->get('decimal_point'),$this->language->get('thousand_point'));

			// Breadcrumbs
			$this->data['breadcrumbs']   = array();
			$this->data['breadcrumbs'][] = array(
				'href'      => $this->url->link('common/home', (isset($this->session->data['token'])) ? 'token=' . $this->session->data['token'] : '', 'SSL'),
				'text'      => $this->language->get('text_home'),
				'separator' => FALSE
			);

			// check if template exists
			if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/bitkassa_bitcoin_return.tpl')) {
				$this->template = $this->config->get('config_template') . '/template/payment/bitkassa_bitcoin_return.tpl';
			} else {
				$this->template = 'default/template/payment/bitkassa_bitcoin_return.tpl';
			}

			$this->children = array(
				'common/column_left',
				'common/column_right',
				'common/content_top',
				'common/content_bottom',
				'common/footer',
				'common/header'
			);

			// Render HTML output
			$this->response->setOutput($this->render());
		}
	}

	public function getOrderBitcoinPrice($order=false) {
		// Haal bitcoinprijs op
		if ($order === false) {
			$this->load->model('checkout/order');
			$this->load->model('payment/bitkassa_bitcoin');
			$order = $this->model_payment_bitkassa_bitcoin->getOrderById($this->session->data['order_id']);
		}
		if ($order['total'] <= 0) {
			return false;
		}
		$btcjson = json_decode(file_get_contents("https://bitkassa.nl/json/price/?euro=" . $order['total']));
		if (isset($btcjson->error)) {
			return false;
		}
		return $btcjson->btc;
	}
}

?>
